#include <ESP32Servo.h>
#include <Arduino.h>
#include <Wire.h>               // Only needed for Arduino 1.6.5 and earlier
#include "SSD1306Wire.h"        // legacy: #include "SSD1306.h"

SSD1306Wire display(0x3c, SDA, SCL); 
Servo ESC;     // Der ESC-Controller (Electronic Speed Controller bzw. elektronischer Fahrtregler) wird als Objekt mit dem Namen "ESC" festgelegt.
int Drehregler;  // Ausgabewert des Drehreglers
int speed, last_speed; // Das Wort "Geschwindigkeit" steht als Variable für den Ansteuerungswert am ESC.
unsigned long turn = 0;
unsigned long speedturns = 0;
unsigned long speedchanged_ms = 0;
unsigned long speed_u_per_min = 0;

const uint8_t LED_BUILTIN = 2;
const uint8_t POTI_PIN = 4;
const uint8_t ESC_PIN = 16;
const uint8_t LS1_PIN = 14;

const uint8_t MAX_ESC_SPEED = 60;
const uint8_t MIN_ESC_SPEED = 35;
const uint16_t MAX_POTI_VALUE = 4095;

void show_values();
void ISR();

void setup() 
{
  Serial.begin(9600);
   // Initialising the UI will init the display too.
  display.init();
  display.clear();
  display.flipScreenVertically();
  //display.setFont(ArialMT_Plain_10);
  display.setTextAlignment(TEXT_ALIGN_CENTER);
  display.setFont(ArialMT_Plain_24);
  display.setFont(ArialMT_Plain_16);
  display.drawString(64, 2, "Init controller");
  display.setFont(ArialMT_Plain_10);
  display.drawString(64, 40, "You may hear some beeps.");
  display.drawString(64, 52, "That's OK! ;-)");
  display.display();  
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_16);
  ESC.attach(ESC_PIN,1000,2000); 

  for(uint8_t i=0;i<=100;i=i+10)
  {
    display.drawProgressBar(6, 28, 116, 6, i);
    delay(500);
    ESC.write(0);
    display.display();    
  }

  attachInterrupt(LS1_PIN, ISR, RISING);

  last_speed = 0;
  speed = 0;


}

void loop() 
{
  
  Drehregler = analogRead(POTI_PIN);   // Dieser Befehl liest den Wert des Potentiometers am analogen Pin A0 aus und speichert ihn unter der Variablen "Drehregler". Der Wert liegt zwischen 0 und 1023.
  speed = map(Drehregler, 0, MAX_POTI_VALUE, MIN_ESC_SPEED, MAX_ESC_SPEED);   // Der "MAP-" Befehl wandelt den Messwert aus der Variablen "Drehregler" um, damit er am ESC verarbeitet werden kann. Der Zahlenbereich 0 bis 1023 wird dabei in einen Zahlenwert zwischen 0 und 180 umgewandelt.
  if(speed != last_speed)
  {
    last_speed = speed;
    speedchanged_ms = millis();
    speedturns = 0;
  }

  if(millis() - speedchanged_ms > 10000)
  {
    speed_u_per_min = (speedturns*6);
    speedturns = 0;
    speedchanged_ms = millis();
  }

  //Serial.print("Drehregler:"); Serial.print(Drehregler); Serial.print(",  Speed:");Serial.println(speed);
  if(MIN_ESC_SPEED < speed)
    ESC.write(speed);    // Der endgültige Wert für den ESC wird an den ESC gesendet. Der ESC nimmt das Signal an dieser Stelle auf und steuert den Motor entsprechend der gesendeten Werte an.
  else
    ESC.write(0); 

  show_values();
}

void show_values()
{
    display.clear();
    display.setTextAlignment(TEXT_ALIGN_LEFT);
    display.setFont(ArialMT_Plain_10);
    display.drawString(1, 1, "Speed:");
    display.drawProgressBar(40, 4, 80, 6, map(speed, MIN_ESC_SPEED,MAX_ESC_SPEED,0,100));
    display.setFont(ArialMT_Plain_10);
    display.drawString(1, 15, "Turns:");
    display.drawString(35, 15, String(turn));
    display.setFont(ArialMT_Plain_24);
    display.setTextAlignment(TEXT_ALIGN_RIGHT);
    display.drawString(64, 37, String(speed_u_per_min));
    display.setTextAlignment(TEXT_ALIGN_LEFT);
    display.drawString(68,37, "U/min");
    display.display();    
}

void IRAM_ATTR ISR()
{
  turn++;
  speedturns++;
}


